@@ -2450,7 +2450,7 @@ IRState::integerConstant(xdmd_integer_t value, tree type)
     tree tree_value = build_int_2(value & 0xffffffff, (value >> 32) & 0xffffffff);
 # elif HOST_BITS_PER_WIDE_INT == 64
     tree tree_value = build_int_2(value,
-	type && ! TREE_UNSIGNED(type) && (value & 0x8000000000000000ULL) ?
+	type && ! TYPE_UNSIGNED(type) && (value & 0x8000000000000000ULL) ?
 	~(unsigned HOST_WIDE_INT) 0 : 0);
 # else
 #  error Fix This
@@ -2538,6 +2538,15 @@ IRState::getFrameForSymbol(Dsymbol * nested_sym)
 	// gcc_assert(nested_func->isNested())
 	outer_func = nested_func->toParent2()->isFuncDeclaration();
 	gcc_assert(outer_func != NULL);
+
+	if (func != outer_func)
+	{
+	    if (!func->vthis) // if no frame pointer for this function
+	    {
+		nested_func->error("is a nested function and cannot be accessed from %s", func->toChars());
+		return d_null_pointer;
+	    }
+	}
     }
     else
     {
@@ -2592,7 +2601,7 @@ IRState::getFrameForSymbol(Dsymbol * nested_sym)
 	    if (! o) {
 	    cannot_access_frame:
 		error("cannot access frame of function '%s' from '%s'",
-		    outer_func->ident->string, func->ident->string);
+		      outer_func->toChars(), func->toChars());
 		return d_null_pointer;
 	    }
 	}
@@ -2894,7 +2903,8 @@ IRState::endCond() { expand_end_cond(); }
 
 void
 IRState::startLoop(Statement * stmt) {
-    beginFlow(stmt, expand_start_loop_continue_elsewhere(1));
+    Flow *f = beginFlow(stmt, expand_start_loop_continue_elsewhere(1));
+    f->continueLabel = label(stmt ? stmt->loc : 0);
 }
 
 void
@@ -2926,7 +2936,8 @@ IRState::startCase(Statement * stmt, tree t_cond)
     clear_last_expr ();
     g.ofile->doLineNote(stmt->loc);
     expand_start_case( 1, t_cond, TREE_TYPE( t_cond ), "switch statement" );
-    beginFlow(stmt, NULL);
+    Flow * f = beginFlow(stmt, NULL);
+    f->condition = t_cond;
 }
 
 void
@@ -3071,7 +3082,7 @@ IRState::startCond(Statement * stmt, Expression * e_cond)
 {
     tree t_cond = convertForCondition(e_cond);
 
-    Flow * f = beginFlow(stmt/*, 0*/);
+    Flow * f = beginFlow(stmt);
     f->condition = t_cond;
 }
 
@@ -3103,7 +3114,7 @@ IRState::endCond()
 
 void
 IRState::startLoop(Statement * stmt) {
-    Flow * f = beginFlow(stmt/*, Break|Continue*/);
+    Flow * f = beginFlow(stmt);
     f->continueLabel = label(stmt ? stmt->loc : 0); // should be end for 'do' loop
 }
 
