@@ -1,6 +1,9 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
    
+   Modified by
+    Michael Parrot, (C) 2009, 2010
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -152,8 +155,11 @@ struct IRState : IRBase
     
     tree convertForAssignment(Expression * exp, Type * target_type);
     tree convertForAssignment(tree exp_tree, Type * exp_type, Type * target_type);
-
+#if V2 //NOTE: When the dmd frontend for D2 is up to 2.037, the #if V2 can be taken out
     tree convertForArgument(Expression * exp, Argument * arg);
+#else
+    tree convertForArgument(Expression * exp, Parameter * arg);
+#endif
     tree convertForCondition(Expression * exp) {
 	return convertForCondition(exp->toElem(this), exp->type); }
     tree convertForCondition(tree exp_tree, Type * exp_type);
@@ -188,8 +194,13 @@ struct IRState : IRBase
     // Routines to handle variables that are references.
     static bool isDeclarationReferenceType(Declaration * decl);
     static tree trueDeclarationType(Declaration * decl);
+#if V2 //Until 2.037
     static bool isArgumentReferenceType(Argument * arg);
     static tree trueArgumentType(Argument * arg);
+#else
+    static bool isArgumentReferenceType(Parameter * arg);
+    static tree trueArgumentType(Parameter * arg);
+#endif
 
     static tree arrayType(Type * d_type, uinteger_t size) // %% use of integer_t
     { return arrayType(d_type->toCtype(), size); }
