@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -105,7 +105,10 @@ void Declaration::checkModify(Loc loc, Scope *sc, Type *t)
     if (sc->incontract && isParameter())
         error(loc, "cannot modify parameter '%s' in contract", toChars());
 
-    if (isCtorinit())
+    if (sc->incontract && isResult())
+        error(loc, "cannot modify result '%s' in contract", toChars());
+
+    if (isCtorinit() && !t->isMutable())
     {   // It's only modifiable if inside the right constructor
         Dsymbol *s = sc->func;
         while (1)
@@ -489,11 +492,11 @@ void AliasDeclaration::semantic(Scope *sc)
 
 #if DMDV2
     type = type->addStorageClass(storage_class);
-    if (storage_class & (STCref | STCnothrow | STCpure))
+    if (storage_class & (STCref | STCnothrow | STCpure | STCdisable))
     {   // For 'ref' to be attached to function types, and picked
         // up by Type::resolve(), it has to go into sc.
         sc = sc->push();
-        sc->stc |= storage_class & (STCref | STCnothrow | STCpure | STCshared);
+        sc->stc |= storage_class & (STCref | STCnothrow | STCpure | STCshared | STCdisable);
         type->resolve(loc, sc, &e, &t, &s);
         sc = sc->pop();
     }
@@ -533,7 +536,7 @@ void AliasDeclaration::semantic(Scope *sc)
     //printf("alias is a symbol %s %s\n", s->kind(), s->toChars());
     type = NULL;
     VarDeclaration *v = s->isVarDeclaration();
-    if (v && v->linkage == LINKdefault)
+    if (0 && v && v->linkage == LINKdefault)
     {
         error("forward reference of %s", v->toChars());
         s = NULL;
@@ -561,7 +564,7 @@ void AliasDeclaration::semantic(Scope *sc)
             s = NULL;
         }
     }
-    if (!aliassym)
+    //printf("setting aliassym %s to %s %s\n", toChars(), s->kind(), s->toChars());
     aliassym = s;
     this->inSemantic = 0;
 }
@@ -624,8 +627,10 @@ Dsymbol *AliasDeclaration::toAlias()
     //static int count; if (++count == 75) exit(0); //*(char*)0=0;
     if (inSemantic)
     {   error("recursive alias declaration");
-                aliassym = new TypedefDeclaration(loc, ident, Type::terror, NULL);
+        aliassym = new TypedefDeclaration(loc, ident, Type::terror, NULL);
     }
+    else if (!aliassym && scope)
+        semantic(scope);
     Dsymbol *s = aliassym ? aliassym->toAlias() : this;
     return s;
 }
@@ -683,6 +688,9 @@ VarDeclaration::VarDeclaration(Loc loc, Type *type, Identifier *id, Initializer
     this->loc = loc;
     offset = 0;
     noauto = 0;
+#if DMDV2
+    isargptr = FALSE;
+#endif
 #if DMDV1
     nestedref = 0;
 #endif
@@ -830,8 +838,8 @@ void VarDeclaration::semantic(Scope *sc)
             OutBuffer buf;
             buf.printf("_%s_field_%"PRIuSIZE, ident->toChars(), i);
             buf.writeByte(0);
-            char *name = (char *)buf.extractData();
-            Identifier *id = new Identifier(name, TOKidentifier);
+            const char *name = (const char *)buf.extractData();
+            Identifier *id = Lexer::idPool(name);
 
             Expression *einit = ie;
             if (ie && ie->op == TOKtuple)
@@ -1329,6 +1337,9 @@ void VarDeclaration::checkNestedReference(Scope *sc, Loc loc)
             nestedref = 1;
             fdv->nestedFrameRef = 1;
             //printf("var %s in function %s is nested ref\n", toChars(), fdv->toChars());
+            // __dollar creates problems because it isn't a real variable Bugzilla 3326
+            if (ident == Id::dollar)
+                ::error(loc, "cannnot use $ inside a function literal");
         }
     }
 }
@@ -1373,7 +1384,7 @@ int VarDeclaration::isCTFE()
 {
     //printf("VarDeclaration::isCTFE(%p, '%s')\n", this, toChars());
     //printf("%llx\n", storage_class);
-    return (storage_class & STCctfe) || !isDataseg();
+    return (storage_class & STCctfe) != 0; // || !isDataseg();
 }
 
 int VarDeclaration::hasPointers()
