@@ -17,19 +17,18 @@
 // Program to generate string files in d data structures.
 // Saves much tedious typing, and eliminates typo problems.
 // Generates:
-//	id.h
-//	id.c
+//      id.h
+//      id.c
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
 #include <assert.h>
 
 struct Msgtable
 {
-	const char *ident;	// name to use in DMD source
-	const char *name;	// name in D executable
+        const char *ident;      // name to use in DMD source
+        const char *name;       // name in D executable
 };
 
 struct Msgtable msgtable[] =
@@ -44,10 +43,12 @@ struct Msgtable msgtable[] =
     { "dtor", "_dtor" },
     { "classInvariant", "__invariant" },
     { "unitTest", "_unitTest" },
+    { "require", "__require" },
+    { "ensure", "__ensure" },
     { "init" },
     { "size" },
     { "__sizeof", "sizeof" },
-    { "alignof" },
+    { "__xalignof", "alignof" },
     { "mangleof" },
     { "stringof" },
     { "tupleof" },
@@ -143,7 +144,7 @@ struct Msgtable msgtable[] =
     { "__LOCAL_SIZE" },
 
     // For operator overloads
-    { "uadd",	 "opPos" },
+    { "uadd",    "opPos" },
     { "neg",     "opNeg" },
     { "com",     "opCom" },
     { "add",     "opAdd" },
@@ -187,14 +188,14 @@ struct Msgtable msgtable[] =
     { "catass",  "opCatAssign" },
     { "postinc", "opPostInc" },
     { "postdec", "opPostDec" },
-    { "index",	 "opIndex" },
+    { "index",   "opIndex" },
     { "indexass", "opIndexAssign" },
-    { "slice",	 "opSlice" },
+    { "slice",   "opSlice" },
     { "sliceass", "opSliceAssign" },
-    { "call",	 "opCall" },
-    { "cast",	 "opCast" },
-    { "match",	 "opMatch" },
-    { "next",	 "opNext" },
+    { "call",    "opCall" },
+    { "cast",    "opCast" },
+    { "match",   "opMatch" },
+    { "next",    "opNext" },
     { "opIn" },
     { "opIn_r" },
 
@@ -213,6 +214,10 @@ struct Msgtable msgtable[] =
     { "aaKeys", "_aaKeys" },
     { "aaValues", "_aaValues" },
     { "aaRehash", "_aaRehash" },
+    { "monitorenter", "_d_monitorenter" },
+    { "monitorexit", "_d_monitorexit" },
+    { "criticalenter", "_d_criticalenter" },
+    { "criticalexit", "_d_criticalexit" },
 
     // For pragma's
     { "GNU_asm" },
@@ -239,71 +244,71 @@ int main()
     unsigned i;
 
     {
-	fp = fopen("id.h","w");
-	if (!fp)
-	{   printf("can't open id.h\n");
-	    exit(EXIT_FAILURE);
-	}
+        fp = fopen("id.h","w");
+        if (!fp)
+        {   printf("can't open id.h\n");
+            exit(EXIT_FAILURE);
+        }
 
-	fprintf(fp, "// File generated by idgen.c\n");
+        fprintf(fp, "// File generated by idgen.c\n");
 #if __DMC__
-	fprintf(fp, "#pragma once\n");
+        fprintf(fp, "#pragma once\n");
 #endif
-	fprintf(fp, "#ifndef DMD_ID_H\n");
-	fprintf(fp, "#define DMD_ID_H 1\n");
-	fprintf(fp, "struct Identifier;\n");
-	fprintf(fp, "struct Id\n");
-	fprintf(fp, "{\n");
+        fprintf(fp, "#ifndef DMD_ID_H\n");
+        fprintf(fp, "#define DMD_ID_H 1\n");
+        fprintf(fp, "struct Identifier;\n");
+        fprintf(fp, "struct Id\n");
+        fprintf(fp, "{\n");
 
-	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   const char *id = msgtable[i].ident;
+        for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
+        {   const char *id = msgtable[i].ident;
 
-	    fprintf(fp,"    static Identifier *%s;\n", id);
-	}
+            fprintf(fp,"    static Identifier *%s;\n", id);
+        }
 
-	fprintf(fp, "    static void initialize();\n");
-	fprintf(fp, "};\n");
-	fprintf(fp, "#endif\n");
+        fprintf(fp, "    static void initialize();\n");
+        fprintf(fp, "};\n");
+        fprintf(fp, "#endif\n");
 
-	fclose(fp);
+        fclose(fp);
     }
 
     {
-	fp = fopen("id.c","w");
-	if (!fp)
-	{   printf("can't open id.c\n");
-	    exit(EXIT_FAILURE);
-	}
+        fp = fopen("id.c","w");
+        if (!fp)
+        {   printf("can't open id.c\n");
+            exit(EXIT_FAILURE);
+        }
 
-	fprintf(fp, "// File generated by idgen.c\n");
-	fprintf(fp, "#include \"id.h\"\n");
-	fprintf(fp, "#include \"identifier.h\"\n");
-	fprintf(fp, "#include \"lexer.h\"\n");
+        fprintf(fp, "// File generated by idgen.c\n");
+        fprintf(fp, "#include \"id.h\"\n");
+        fprintf(fp, "#include \"identifier.h\"\n");
+        fprintf(fp, "#include \"lexer.h\"\n");
 
-	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   const char *id = msgtable[i].ident;
- 	    const char *p = msgtable[i].name;
+        for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
+        {   const char *id = msgtable[i].ident;
+            const char *p = msgtable[i].name;
 
-	    if (!p)
-		p = id;
-	    fprintf(fp,"Identifier *Id::%s;\n", id);
-	}
+            if (!p)
+                p = id;
+            fprintf(fp,"Identifier *Id::%s;\n", id);
+        }
 
-	fprintf(fp, "void Id::initialize()\n");
-	fprintf(fp, "{\n");
+        fprintf(fp, "void Id::initialize()\n");
+        fprintf(fp, "{\n");
 
-	for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
-	{   const char *id = msgtable[i].ident;
- 	    const char *p = msgtable[i].name;
+        for (i = 0; i < sizeof(msgtable) / sizeof(msgtable[0]); i++)
+        {   const char *id = msgtable[i].ident;
+            const char *p = msgtable[i].name;
 
-	    if (!p)
-		p = id;
-	    fprintf(fp,"    %s = Lexer::idPool(\"%s\");\n", id, p);
-	}
+            if (!p)
+                p = id;
+            fprintf(fp,"    %s = Lexer::idPool(\"%s\");\n", id, p);
+        }
 
-	fprintf(fp, "}\n");
+        fprintf(fp, "}\n");
 
-	fclose(fp);
+        fclose(fp);
     }
 
     return EXIT_SUCCESS;
