@@ -54,6 +54,7 @@ Expression *interpret_values(InterState *istate, Expression *earg, FuncDeclarati
 
 ArrayLiteralExp *createBlockDuplicatedArrayLiteral(Type *type, Expression *elem, size_t dim);
 Expression * resolveReferences(Expression *e, Expression *thisval, bool *isReference = NULL);
+Expression *getVarExp(Loc loc, InterState *istate, Declaration *d);
 
 /*************************************
  * Attempt to interpret a function given the arguments.
@@ -129,6 +130,11 @@ Expression *FuncDeclaration::interpret(InterState *istate, Expressions *argument
         error("need 'this' to access member %s", toChars());
         return NULL;
     }
+    if (thisarg && !istate)
+    {   // Check that 'this' aleady has a value
+        if (thisarg->interpret(istate) == EXP_CANT_INTERPRET)
+            return NULL;
+    }
     if (arguments)
     {
         dim = arguments->dim;
@@ -145,7 +151,15 @@ Expression *FuncDeclaration::interpret(InterState *istate, Expressions *argument
         {   Expression *earg = (Expression *)arguments->data[i];
             Parameter *arg = Parameter::getNth(tf->parameters, i);
 
-            if (arg->storageClass & (STCout | STCref | STClazy))
+            if (arg->storageClass & (STCout | STCref))
+            {
+                if (!istate)
+                {
+                    earg->error("%s cannot be passed by reference at compile time", earg->toChars());
+                    return NULL;
+                }
+            }
+            else if (arg->storageClass & STClazy)
             {
             }
             else
@@ -159,11 +173,11 @@ Expression *FuncDeclaration::interpret(InterState *istate, Expressions *argument
                      */
                     earg = ((AddrExp *)earg)->e1;
                 }
-                earg = earg->interpret(istate ? istate : &istatex);
+                earg = earg->interpret(istate); // ? istate : &istatex);
                 if (earg == EXP_CANT_INTERPRET)
                 {   cantInterpret = 1;
                     return NULL;
-            }
+                }
             }
             eargs.data[i] = earg;
         }
@@ -589,7 +603,7 @@ Expression *WhileStatement::interpret(InterState *istate)
     printf("WhileStatement::interpret()\n");
 #endif
     assert(0);                  // rewritten to ForStatement
-        return NULL;
+    return NULL;
 }
 
 Expression *DoStatement::interpret(InterState *istate)
@@ -853,10 +867,10 @@ Expression *ForeachRangeStatement::interpret(InterState *istate)
             }
             if (e == NULL || e == EXP_CONTINUE_INTERPRET)
             {   e = Add(key->value->type, key->value, new IntegerExp(loc, 1, key->value->type));
-            if (e == EXP_CANT_INTERPRET)
-                break;
-            key->value = e;
-        }
+                if (e == EXP_CANT_INTERPRET)
+                    break;
+                key->value = e;
+            }
             else
                 break;
         }
@@ -1019,6 +1033,7 @@ Expression *LabelStatement::interpret(InterState *istate)
     return statement ? statement->interpret(istate) : NULL;
 }
 
+
 Expression *TryCatchStatement::interpret(InterState *istate)
 {
 #if LOG
@@ -1095,8 +1110,9 @@ Expression *Expression::interpret(InterState *istate)
 
 Expression *ThisExp::interpret(InterState *istate)
 {
-    if (istate->localThis)
+    if (istate && istate->localThis)
         return istate->localThis->interpret(istate);
+    error("value of 'this' is not known at compile time");
     return EXP_CANT_INTERPRET;
 }
 
@@ -1235,7 +1251,7 @@ Expression *getVarExp(Loc loc, InterState *istate, Declaration *d)
             if (e && !e->type)
                 e->type = v->type;
         }
-        else if (v->isCTFE() && !v->value)
+        else if ((v->isCTFE() || (!v->isDataseg() && istate)) && !v->value)
         {
             if (v->init)
             {
@@ -1250,9 +1266,13 @@ Expression *getVarExp(Loc loc, InterState *istate, Declaration *d)
             else
                 e = v->type->defaultInitLiteral(loc);
         }
+        else if (!v->isDataseg() && !istate) {
+            error(loc, "variable %s cannot be read at compile time", v->toChars());
+            return EXP_CANT_INTERPRET;
+        }
         else
         {   e = v->value;
-            if (!v->isCTFE())
+             if (!v->isCTFE() && v->isDataseg())
             {   error(loc, "static variable %s cannot be read at compile time", v->toChars());
                 e = EXP_CANT_INTERPRET;
             }
@@ -1386,7 +1406,7 @@ Expression *ArrayLiteralExp::interpret(InterState *istate)
 
             ex = e->interpret(istate);
             if (ex == EXP_CANT_INTERPRET)
-            goto Lerror;
+                goto Lerror;
 
             /* If any changes, do Copy On Write
              */
@@ -1518,7 +1538,7 @@ Expression *StructLiteralExp::interpret(InterState *istate)
     /* We don't know how to deal with overlapping fields
      */
     if (sd->hasUnions)
-        {   error("Unions with overlapping fields are not yet supported in CTFE");
+    {   error("Unions with overlapping fields are not yet supported in CTFE");
         return EXP_CANT_INTERPRET;
     }
 
@@ -1576,7 +1596,8 @@ Expression *NewExp::interpret(InterState *istate)
         if (lenExpr == EXP_CANT_INTERPRET)
             return EXP_CANT_INTERPRET;
         return createBlockDuplicatedArrayLiteral(newtype,
-            newtype->defaultInitLiteral(), lenExpr->toInteger());
+            ((TypeArray *)newtype)->next->defaultInitLiteral(),
+            lenExpr->toInteger());
     }
     error("Cannot interpret %s at compile time", toChars());
     return EXP_CANT_INTERPRET;
@@ -1844,7 +1865,7 @@ Expression * modifyStructField(Type *type, StructLiteralExp *se, size_t offset,
  * set arr[index] = newval and return the new array.
  *
  */
-Expression * assignArrayElement(Expression *arr, Expression *index, Expression *newval)
+Expression * assignArrayElement(Loc loc, Expression *arr, Expression *index, Expression *newval)
 {
         ArrayLiteralExp *ae = NULL;
         AssocArrayLiteralExp *aae = NULL;
@@ -1862,8 +1883,8 @@ Expression * assignArrayElement(Expression *arr, Expression *index, Expression *
             int elemi = index->toInteger();
             if (elemi >= ae->elements->dim)
             {
-                error("array index %d is out of bounds %s[0..%d]", elemi,
-                        arr->toChars(), ae->elements->dim);
+                error(loc, "array index %d is out of bounds %s[0..%d]", elemi,
+                    arr->toChars(), ae->elements->dim);
                 return EXP_CANT_INTERPRET;
             }
             // Create new array literal reflecting updated elem
@@ -1877,6 +1898,12 @@ Expression * assignArrayElement(Expression *arr, Expression *index, Expression *
             /* Create new string literal reflecting updated elem
              */
             int elemi = index->toInteger();
+            if (elemi >= se->len)
+            {
+                error(loc, "array index %d is out of bounds %s[0..%d]", elemi,
+                    arr->toChars(), se->len);
+                return EXP_CANT_INTERPRET;
+            }
             unsigned char *s;
             s = (unsigned char *)mem.calloc(se->len + 1, se->sz);
             memcpy(s, se->string, se->len * se->sz);
@@ -2076,6 +2103,11 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
 #endif
     Expression *e = EXP_CANT_INTERPRET;
     Expression *e1 = this->e1;
+    if (!istate)
+    {
+        error("value of %s is not known at compile time", e1->toChars());
+        return e;
+    }
 
     if (fp)
     {
@@ -2087,6 +2119,8 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
     if (e1 == EXP_CANT_INTERPRET)
         return e1;
 
+    assert(istate);
+
     // ----------------------------------------------------
     //  Deal with read-modify-write assignments.
     //  Set 'newval' to the final assignment value
@@ -2097,7 +2131,7 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
     if (newval == EXP_CANT_INTERPRET)
         return newval;
 
-    if (fp)
+    if (fp || e1->op == TOKarraylength)
     {
         // If it isn't a simple assignment, we need the existing value
         Expression * oldval = e1->interpret(istate);
@@ -2105,17 +2139,72 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
             return EXP_CANT_INTERPRET;
         while (oldval->op == TOKvar)
         {
-          oldval = resolveReferences(oldval, istate->localThis);
-          oldval = oldval->interpret(istate);
-          if (oldval == EXP_CANT_INTERPRET) return oldval;
+            oldval = resolveReferences(oldval, istate->localThis);
+            oldval = oldval->interpret(istate);
+            if (oldval == EXP_CANT_INTERPRET)
+                return oldval;
+        }
+
+        if (fp)
+        {
+            newval = (*fp)(type, oldval, newval);
+            if (newval == EXP_CANT_INTERPRET)
+            {
+                error("Cannot interpret %s at compile time", toChars());
+                return EXP_CANT_INTERPRET;
+            }
+            // Determine the return value
+            e = Cast(type, type, post ? oldval : newval);
+            if (e == EXP_CANT_INTERPRET)
+                return e;
+        }
+        else
+            e = newval;
+
+        if (e1->op == TOKarraylength)
+        {
+            size_t oldlen = oldval->toInteger();
+            size_t newlen = newval->toInteger();
+            if (oldlen == newlen) // no change required -- we're done!
+                return e;
+            // Now change the assignment from arr.length = n into arr = newval
+            e1 = ((ArrayLengthExp *)e1)->e1;
+            if (oldlen != 0)
+            {   // Get the old array literal.
+                oldval = e1->interpret(istate);
+                while (oldval->op == TOKvar)
+                {   oldval = resolveReferences(oldval, istate->localThis);
+                    oldval = oldval->interpret(istate);
+                }
+            }
+            Type *t = e1->type->toBasetype();
+            if (t->ty == Tarray)
+            {
+                Type *elemType= NULL;
+                elemType = ((TypeArray *)t)->next;
+                assert(elemType);
+                Expression *defaultElem = elemType->defaultInitLiteral();
+
+                Expressions *elements = new Expressions();
+                elements->setDim(newlen);
+                size_t copylen = oldlen < newlen ? oldlen : newlen;
+                ArrayLiteralExp *ae = (ArrayLiteralExp *)oldval;
+                for (size_t i = 0; i < copylen; i++)
+                     elements->data[i] = ae->elements->data[i];
+
+                for (size_t i = copylen; i < newlen; i++)
+                    elements->data[i] = defaultElem;
+                ArrayLiteralExp *aae = new ArrayLiteralExp(0, elements);
+                aae->type = t;
+                newval = aae;
+            }
+            else
+            {
+                error("%s is not yet supported at compile time", toChars());
+                return EXP_CANT_INTERPRET;
+            }
+
         }
-        newval = (*fp)(type, oldval, newval);
-        if (newval == EXP_CANT_INTERPRET)
-            return EXP_CANT_INTERPRET;
-        // Determine the return value
-        e = Cast(type, type, post ? oldval : newval);
-        if (e == EXP_CANT_INTERPRET)
-            return e;
     }
     else if (e1->op != TOKslice)
     {   /* Look for special case of struct being initialized with 0.
@@ -2136,7 +2225,7 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
     // Make sure we're not trying to modify a global or static variable
     // We do this by locating the ultimate parent variable which gets modified.
     VarDeclaration * ultimateVar = findParentVar(e1, istate->localThis);
-    if (ultimateVar && !ultimateVar->isCTFE())
+    if (ultimateVar && ultimateVar->isDataseg() && !ultimateVar->isCTFE())
     {   // Can't modify global or static data
         error("%s cannot be modified at compile time", ultimateVar->toChars());
         return EXP_CANT_INTERPRET;
@@ -2283,7 +2372,9 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
             Expression *index = ie->e2->interpret(istate);
             if (index == EXP_CANT_INTERPRET)
                 return EXP_CANT_INTERPRET;
-            newval = assignArrayElement(v->value, index, newval);
+            newval = assignArrayElement(loc, v->value, index, newval);
+            if (newval == EXP_CANT_INTERPRET)
+                return EXP_CANT_INTERPRET;
             v->value = newval;
             return e;
         }
@@ -2444,7 +2535,7 @@ Expression *BinExp::interpretAssignCommon(InterState *istate, fp_t fp, int post)
         if (((PtrExp *)e1)->e1->op == TOKsymoff)
         {   SymOffExp *soe = (SymOffExp *)((PtrExp *)e1)->e1;
             VarDeclaration *v = soe->var->isVarDeclaration();
-            if (!v->isCTFE())
+            if (v->isDataseg() && !v->isCTFE())
             {
                 error("%s cannot be modified at compile time", v->toChars());
                 return EXP_CANT_INTERPRET;
@@ -2714,7 +2805,7 @@ Expression *CallExp::interpret(InterState *istate)
             {
                 e = expType(type, e);
             }
-            }
+        }
         else
         {
             if (!fd->fbody)
@@ -2745,17 +2836,22 @@ Expression *CommaExp::interpret(InterState *istate)
 #if LOG
     printf("CommaExp::interpret() %s\n", toChars());
 #endif
+
+    CommaExp * firstComma = this;
+    while (firstComma->e1->op == TOKcomma)
+        firstComma = (CommaExp *)firstComma->e1;
+
+    // If it creates a variable, and there's no context for
+    // the variable to be created in, we need to create one now.
+    InterState istateComma;
+    if (!istate &&  firstComma->e1->op == TOKdeclaration)
+        istate = &istateComma;
+
     // If the comma returns a temporary variable, it needs to be an lvalue
     // (this is particularly important for struct constructors)
     if (e1->op == TOKdeclaration && e2->op == TOKvar
        && ((DeclarationExp *)e1)->declaration == ((VarExp*)e2)->var)
     {
-        // If there's no context for the variable to be created in,
-        // we need to create one now.
-        InterState istateComma;
-        if (!istate)
-            istate = &istateComma;
-
         VarExp* ve = (VarExp *)e2;
         VarDeclaration *v = ve->var->isVarDeclaration();
         if (!v->init && !v->value)
@@ -2770,7 +2866,6 @@ Expression *CommaExp::interpret(InterState *istate)
             v->value = newval;
         return e2;
     }
-
     Expression *e = e1->interpret(istate);
     if (e != EXP_CANT_INTERPRET)
         e = e2->interpret(istate);
@@ -2847,7 +2942,10 @@ Expression *IndexExp::interpret(InterState *istate)
     e2 = this->e2->interpret(istate);
     if (e2 == EXP_CANT_INTERPRET)
         goto Lcant;
-    return Index(type, e1, e2);
+    e = Index(type, e1, e2);
+    if (e == EXP_CANT_INTERPRET)
+        error("%s cannot be interpreted at compile time", toChars());
+    return e;
 
 Lcant:
     return EXP_CANT_INTERPRET;
@@ -2889,7 +2987,10 @@ Expression *SliceExp::interpret(InterState *istate)
     if (upr == EXP_CANT_INTERPRET)
         goto Lcant;
 
-    return Slice(type, e1, lwr, upr);
+    e = Slice(type, e1, lwr, upr);
+    if (e == EXP_CANT_INTERPRET)
+        error("%s cannot be interpreted at compile time", toChars());
+    return e;
 
 Lcant:
     return EXP_CANT_INTERPRET;
@@ -2912,7 +3013,10 @@ Expression *CatExp::interpret(InterState *istate)
     e2 = this->e2->interpret(istate);
     if (e2 == EXP_CANT_INTERPRET)
         goto Lcant;
-    return Cat(type, e1, e2);
+    e = Cat(type, e1, e2);
+    if (e == EXP_CANT_INTERPRET)
+        error("%s cannot be interpreted at compile time", toChars());
+    return e;
 
 Lcant:
 #if LOG
@@ -2932,7 +3036,10 @@ Expression *CastExp::interpret(InterState *istate)
     e1 = this->e1->interpret(istate);
     if (e1 == EXP_CANT_INTERPRET)
         goto Lcant;
-    return Cast(type, to, e1);
+    e = Cast(type, to, e1);
+    if (e == EXP_CANT_INTERPRET)
+        error("%s cannot be interpreted at compile time", toChars());
+    return e;
 
 Lcant:
 #if LOG
