@@ -46,6 +46,8 @@ elif grep -q '^4\.2\.' gcc/BASE-VER; then
     gcc_ver=4.2
 elif grep -q '^4\.3\.' gcc/BASE-VER; then
     gcc_ver=4.3
+elif grep -q '^4\.4\.' gcc/BASE-VER; then
+    gcc_ver=4.4
 fi
 
 gcc_patch_key=${gcc_ver}.x
@@ -83,7 +85,27 @@ mkdir libphobos && \
     ../symlink-tree ../gcc/d/phobos$d_subdir_sfx .svn > /dev/null && \
     cd "$top" || exit 1
 
-# 2. Patch the top-level directory
+# 2. If they are building D2, then create links to the druntime sources in
+# the top level directory
+#if test "$d_lang_version" = 2; then
+#    mkdir libdruntime && \
+#    cd libdruntime && \
+#    ../symlink-tree ../gcc/d/druntime .svn > /dev/null && \
+#    cd "$top" || exit 1
+#    
+#    # 2.1. Adding libdruntime to the list of target libs in config-lang.in
+#    cd gcc/d
+#    cat config-lang.in > config-lang.in.tmp
+#    echo "target_libs+=\"target-libdruntime\"" > config-lang.in.tmp2
+#    cat config-lang.in.tmp config-lang.in.tmp2 > config-lang.in.tmp3
+#    cat config-lang.in.tmp3 > config-lang.in
+#    rm config-lang.in.tmp
+#    rm config-lang.in.tmp2
+#    rm config-lang.in.tmp3
+#    cd "$top" || exit 1
+#fi
+
+# 3. Patch the top-level directory
 #
 # If the patch for the top-level Makefile.in doesn't take, you can regenerate
 # it with:
@@ -96,11 +118,11 @@ if test -n "$gcc_apple"; then
     patch -l -p1 < "gcc/d/patches/patch-build_gcc-$gcc_patch_key" || exit 1
 fi
 
-# 3. Patch the gcc subdirectory
+# 4. Patch the gcc subdirectory
 cd gcc || exit 1
 patch -p1 < "$gcc_patch_fn" || exit 1
 
-# 3.1 Patch the gcc version string
+# 4.1 Patch the gcc version string
 if test "$gcc_ver" = 4.1; then
     cur_DEV_PHASE=`cat DEV-PHASE`
     echo "$cur_DEV_PHASE $gdc_ver_msg" > DEV-PHASE
@@ -111,7 +133,7 @@ else
 	version.c > version.c.tmp && mv -f version.c.tmp version.c
 fi
 
-# 4. Maybe apply Darwin patches
+# 5. Maybe apply Darwin patches
 if test -z "$gcc_apple" && test "`uname`" = Darwin; then
     if test -f d/patches/patch-gcc-darwin-eh-$gcc_patch_key; then
 	patch -p1 < d/patches/patch-gcc-darwin-eh-$gcc_patch_key || exit 1
